package org.beliaj.knots.io.impl;

import java.awt.Dimension;
import java.io.*;
import java.util.List;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.*;

import org.beliaj.knots.diagram.*;
import org.beliaj.knots.io.IDiagramWriter;

public class BasicDiagramWriter implements IDiagramWriter {
	
	private String myFileName;
	
	public BasicDiagramWriter(String fileName){
		this.myFileName = fileName;
	}

	public void write(IKnotDiagram kd) throws IOException, ParserConfigurationException, TransformerException{
		write(kd, null);
	}
	
	public void write(IKnotDiagram kd, Dimension dimension) throws IOException, ParserConfigurationException, TransformerException {
		
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance ();
		DocumentBuilder db = dbf.newDocumentBuilder ();
		Document doc = db.newDocument();
		Element root = doc.createElement ("diagram");
		doc.appendChild (root);
		
		
		
		List<IPoint> pList = kd.getPoints();
		List<ILine> lList = kd.getLines();
		
		Element pEl = doc.createElement("points");
		pEl.setAttribute("count", pList.size()+"");
		root.appendChild(pEl);
		
		int i = 0;
		for (IPoint point: pList){
			Element cp = doc.createElement("point");
			cp.setAttribute("id", i+"");
			cp.setAttribute("x",""+point.getX());
			cp.setAttribute("y",""+point.getY());
			i++;
			pEl.appendChild(cp);
		}
		
		i = 0;
		pEl = doc.createElement("lines");
		pEl.setAttribute("count", lList.size()+"");
		root.appendChild(pEl);
		
		Element iEl = doc.createElement("intersections");
		root.appendChild(iEl);
		for (ILine line: lList){
			Element cp = doc.createElement("line");
			cp.setAttribute("id", i+"");
			cp.setAttribute("fpoint", ""+pList.indexOf(line.getSPoint()));
			cp.setAttribute("spoint", ""+pList.indexOf(line.getFPoint()));
			pEl.appendChild(cp);
			
			for (IIntersection ii: line.getIntersections()){
				if (ii.getUpLine().equals(line)){
					cp = doc.createElement("intersection");
					cp.setAttribute("upline", ""+i);
					cp.setAttribute("downline", ""+lList.indexOf(ii.getDownLine()));
					iEl.appendChild(cp);
				}
			}
			
			i++;
		}

		if (dimension != null){
			pEl = doc.createElement("size");
			pEl.setAttribute("width", dimension.width+"");
			pEl.setAttribute("height", dimension.height+"");
			root.appendChild(pEl);
		}
		
		//System.out.println(root.getChildNodes());
		DOMSource domSource = new DOMSource(doc);
		TransformerFactory trFactory = TransformerFactory.newInstance();
		Transformer transformer = trFactory.newTransformer();
		
		transformer.setOutputProperty(OutputKeys.INDENT, "yes");
		
		transformer.transform(domSource, new StreamResult(new FileOutputStream(this.myFileName)));
		
		/*PrintWriter writer = null;
		try{
			writer = new PrintWriter(new File(this.myFileName));
			writer.println("<?xml version=\"1.0\" encoding=\"Windows-1251\"?>");
			writer.println();
			
			writer.println("<diagram>");
			
			List<IPoint> pList = kd.getPoints();
			
			writer.println("<points>");
			int i = 0;
			StringBuffer sb = new StringBuffer();
			for (IPoint point: pList){
				sb.append("<point id = \"");
				sb.append(i);
				sb.append("\" x = \"");
				sb.append(point.getX());
				sb.append("\" y = \"");
				sb.append(point.getY());
				sb.append("\" />");
				writer.println(sb.toString());
				sb.delete(0, sb.length());
				i++;
				if (writer.checkError()) {
					throw new IOException();
				}
			}
			writer.println("</points>");
			
			List<ILine> lList = kd.getLines();
			
			writer.println("<lines>");
			i = 0;
			StringBuffer sb2 = new StringBuffer();
			for (ILine line: lList){
				sb.append("<line id = \"");
				sb.append(i);
				sb.append("\" fpoint = \"");
				sb.append(pList.indexOf(line.getSPoint()));
				sb.append("\" spoint = \"");
				sb.append(pList.indexOf(line.getFPoint()));
				sb.append("\" />");
				writer.println(sb.toString());
				sb.delete(0, sb.length());
				
				
				for (IIntersection ii: line.getIntersections()){
					if (ii.getUpLine().equals(line)){
						sb2.append("<intersection upline = \"");
						sb2.append(i);
						sb2.append("\" downline = \"");
						sb2.append(lList.indexOf(ii.getDownLine()));
						sb2.append("\" />");
						sb2.append('\n');
					}
				}
				
				i++;
				
				if (writer.checkError()) {
					throw new IOException();
				}
			}
			writer.println("</lines>");
			
			writer.println("<intersections>");
			writer.print(sb2.toString());
			writer.println("</intersections>");
			
			if (dimension != null){
				writer.print("<size width = \"");
				writer.print(dimension.width);
				writer.print("\" height =\"");
				writer.print(dimension.height);
				writer.println("\" />");
			}
			
			writer.println("</diagram>");
			
			if (writer.checkError()) {
				throw new IOException();
			}
			
		} finally {
			if (writer != null){	
				writer.close();
			}
		}*/
	}

}
