package org.beliaj.knots.math;

import java.util.Collections;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;

public class DoublePolynom implements Iterable{
	
	private TreeMap<Double, Integer> myTreeMap = new TreeMap<Double, Integer>();

	public DoublePolynom(){
		
	}
	
	protected DoublePolynom(Map<Double, Integer> map) {
		this.myTreeMap.putAll(map);
	}
	
	public DoublePolynom(Polynom p){
		 for(Integer i: p.degreeSet()){
			 this.myTreeMap.put(i.doubleValue(), p.getCoefficient(i));
		 }
	}
	
	public static DoublePolynom createNumber(Integer coeff){
		DoublePolynom p = new DoublePolynom();
		p.myTreeMap.put(0.0, coeff);
		return p;
	}
	
	public static DoublePolynom createMonom(Double degree){
		
		DoublePolynom p = new DoublePolynom();
		p.myTreeMap.put(degree, 1);
		return p;
	}
	
	
	public int getCoefficient(double i){
		Integer d = this.myTreeMap.get(i);
		if (d == null){
			return 0;
		}
		return d;
	}
	
	public DoublePolynom add(DoublePolynom p){
		
		if (p == null){
			return this;
		}
		
		for (Double i: p.myTreeMap.keySet()){
			this.myTreeMap.put(i, this.getCoefficient(i) + p.myTreeMap.get(i));
			if (this.myTreeMap.get(i) == 0){
				this.myTreeMap.remove(i);
			}
		}
		
		return this;
	}
	
	public DoublePolynom mult(double k, int coeff){
		
		if (coeff == 0){
			this.myTreeMap.clear();
			return this;
		}
		
		if (k == 0){
			for (Double i: this.myTreeMap.keySet()){
				this.myTreeMap.put(i, this.myTreeMap.get(i)*coeff);
				if (this.myTreeMap.get(i) == 0){
					this.myTreeMap.remove(i);
				}
			}
		} else {
			TreeMap<Double, Integer> copyOfThis = (TreeMap<Double, Integer>) this.myTreeMap.clone(); 
			this.myTreeMap.clear();
			
			for (Double i: copyOfThis.keySet()){
				this.myTreeMap.put(i+k, copyOfThis.get(i)*coeff);
				if (this.myTreeMap.get(i+k) == 0){
					this.myTreeMap.remove(i+k);
				}
			}
			
		}
		
		
		
		return this;
	}
	
	public DoublePolynom mult(DoublePolynom p){
		
		//System.out.println(this);
		
		if (p == null){
			this.myTreeMap.clear();
			return this;
		}
		
		if (this == p){
			this.degree(2);
			return this;
		}
		
		TreeMap<Double, Integer> copyOfThis = (TreeMap<Double, Integer>) this.myTreeMap.clone(); 
		this.myTreeMap.clear();
		
		for (Double i: p.myTreeMap.keySet()){
			for (Double j: copyOfThis.keySet()){
				this.myTreeMap.put(i+j, this.getCoefficient(i+j) + p.myTreeMap.get(i)*copyOfThis.get(j));
				if (this.myTreeMap.get(i+j) == 0){
					this.myTreeMap.remove(i+j);
				}
			}
		}
		
		return this;
	}
	
	public DoublePolynom degree(int deg){
		
		if (deg < 0) throw new RuntimeException("Degree ( " + deg + " ) is invalid. It must be positive.");
		
		if (deg == 0){
			this.myTreeMap.clear();
			this.myTreeMap.put(0.0, 1); 
			return this;
		}
		
		TreeMap<Double, Integer> copyOfThis = (TreeMap<Double, Integer>) this.myTreeMap.clone(); 
		DoublePolynom p = new DoublePolynom(copyOfThis);
		
		for (int i = 0; i < deg-1; i++){
			this.mult(p);
		}
		
		return this;
	}

	public Iterator iterator() {
		
		return null;
	}
	
	public Set<Double> degreeSet(){
		return Collections.unmodifiableSet(this.myTreeMap.keySet()); 
	}
	
	public String toString(){
		StringBuffer sb = new StringBuffer();
		
		if (this.myTreeMap.size() == 0){
			return "";
		}
		
		Double min = this.myTreeMap.firstKey();
		
		for (Double i: this.myTreeMap.keySet()){
			if (min != i){
				if ((this.myTreeMap.get(i) >= 0)){
					sb.append(" + ");
				} else {
					sb.append(" - ");
				}
			} else if (this.myTreeMap.get(i) < 0){
				sb.append("- ");
			}
			sb.append(Math.abs(this.myTreeMap.get(i)) + " * " + "x^" +  i);
		}
		
		return sb.toString();
	}
	
	public String toHTML(){
		
		//<html>e=mc<sup>2</sup></html>
		StringBuffer sb = new StringBuffer();
		//sb.append("<html>");
		if (this.myTreeMap.size() == 0){
			return "";
		}
		
		Double min = this.myTreeMap.firstKey();
		
		for (Double i: this.myTreeMap.keySet()){
			if (min != i){
				if ((this.myTreeMap.get(i) >= 0)){
					sb.append(" + ");
				} else {
					sb.append(" - ");
				}
			} else if (this.myTreeMap.get(i) < 0){
				sb.append("- ");
			}
			if (Math.abs(this.myTreeMap.get(i)) != 1){
				sb.append(Math.abs(this.myTreeMap.get(i)));
			}
			if (i == 0){
				//sb.append(" ");
				//sb.append("1");
			} else {
				sb.append(" " + "x<sup>" +  (i) + "</sup>");
			}
		}
		
		//sb.append("</html>");
		return sb.toString();
	}
	
	public DoublePolynom substitute(double degree){
		TreeMap<Double, Integer> copyOfThis = (TreeMap<Double, Integer>) this.myTreeMap.clone(); 
		this.myTreeMap.clear();
		
		if (degree == 0){
			return this;
		}
		
		for (Double d: copyOfThis.keySet()){
			this.myTreeMap.put(d*degree, copyOfThis.get(d));
		}
		
		return this;
	}
}
