package org.beliaj.knots.simple;

import java.util.*;

import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.beliaj.knots.diagram.*;
import org.beliaj.knots.diagram.impl.DiagramChangesEnum;
import org.beliaj.knots.math.DoublePolynom;
import org.beliaj.knots.math.Polynom;

/**
 * 
 */
public class SimpleKnot {
	
	/**
	 * ,   
	 */
	private IKnotDiagram myDiagram;
	
	/**
	 *   IIntersection-  SimpleIntersection-
	 */
	private HashMap<IIntersection, SimpleIntersection> myIntersections =
			new HashMap<IIntersection, SimpleIntersection>();
	
	/**
	 *    
	 */
	private Set<SimpleIntersection> mySet = new HashSet<SimpleIntersection>();
	
	/**
	 *     , ..    
	 */
	private int myRingsCount = 0;
	
	/**
	 * 
	 */
	private int myTmpRingsCount = 0;
	
	/**
	 *   
	 */
	private int number = 0;
	
	/**
	 *   
	 */
	private int count = 0;
	
	/**
	 *   -  
	 */
	private boolean hasChanges;
	
	/**
	 *    
	 */
	private Polynom myKaufman;
	
	/**
	 *  -   
	 */
	private Polynom myJonesKaufman;
	
	private DoublePolynom myJones;
	
	private List<ChangeListener> myChangeListeners = new LinkedList<ChangeListener>();
	
	/**
	 * 
	 */
	public SimpleKnot(IKnotDiagram kd){
		this.myDiagram = kd;
		
		if (!kd.isComplete()){
			throw new RuntimeException("KnotDiagram is not complete!");
		}
		
		this.myDiagram.addChangeListener(new DiagramChangeListener());
		
		this.build();
	}
	
	/**
	 * ,       
	 */
	private class DiagramChangeListener implements ChangeListener{

		public void stateChanged(ChangeEvent e) {
			DiagramChangesEnum de = (DiagramChangesEnum) e.getSource();
			if (!de.equals(DiagramChangesEnum.MOVE)){
				SimpleKnot.this.sendChangeEvent(new ChangeEvent(SimpleKnot.this));
				SimpleKnot.this.hasChanges  = true;
				if (!de.equals(DiagramChangesEnum.SHANGE_DIRECTION)){
					SimpleKnot.this.myKaufman = null;
				}
				SimpleKnot.this.myJonesKaufman = null;
				SimpleKnot.this.myJones = null;
				//System.out.println("ch event "+de);
			}
		}
		
	}
	
	/**
	 *    
	 */
	private void build(){
		//System.out.println("rebuild");
		//     
		Set<ILine> marks = new HashSet<ILine>();
		
		ListIterator<ILine> it = this.myDiagram.getLines().listIterator();
		
		this.myIntersections.clear();
		this.myRingsCount = 0;
		this.mySet.clear();
		this.myTmpRingsCount = 0;
		this.number = 0;
		this.myTmpRingsCount = 0;
		
		while(true){
			ILine line = null;
			//   
			while (it.hasNext()){
				line = it.next();
				if (!marks.contains(line)){
					break;
				} else {
					line = null;
				}
			}
			
			//  
			if (line == null){
				break;
			}
			
			//   ,     
			boolean b = false;
			//   
			SimpleIntersection last = null;
			SimpleIntersection first = null;
			//     
			int lastNumber = 0;
			int firstNumber = 0;
			
			//    
			while (true){
				//System.out.println("line: "+line+"\n");
				//    ,       
				if (marks.contains(line)){
					if (first != null){
						first.set(firstNumber, last, lastNumber);
						//System.out.println("set to: "+first.myName);
						//System.out.println("	set1: firstNumber "+ firstNumber + " last " +
						//		last.myName + " lastNumber " + lastNumber);
						
						last.set(lastNumber, first, firstNumber);
						//System.out.println("set to: "+last.myName);
						//System.out.println("	set2: lastNumber "+ lastNumber +
						//		" first " + first.myName + " firstNumber " + firstNumber);
						first = null;
						
					}
					break;
				}
				
				//   
				List<IIntersection> list = line.getIntersections();
				if (list.size() > 0){
					
					//    -   
					// ,   ,     KnotDiagram
					//    
					TreeSet<SimpleIntersection> set = new TreeSet<SimpleIntersection>(new IntersectionComparator()); 
					
					//  ,  simpleintersection-      set
					for (IIntersection ii: list){
						SimpleIntersection si;
						if (this.myIntersections.containsKey(ii)){
							si = this.myIntersections.get(ii);
							set.add(si);
						} else {
							si = new SimpleIntersection(ii, this.number++);
							set.add(si);
							this.myIntersections.put(ii, si);
						}
					}
					
					//System.out.println("set: " + set);
					//System.out.println("hashmap: " + this.myIntersections);
					
					// , ,     ,    
					// ,  
					for (SimpleIntersection si: set){
						//System.out.println("si: " + si.myName);
						
						if (last != null){
							last.set(lastNumber, si, si.getFNumber(line));
							si.set(si.getFNumber(line), last, lastNumber);
							//System.out.println("set to: "+last.myName);
							//System.out.println("	set: lastNumber "+ lastNumber + " si " +
							//		si.myName + " si.getFNumber(line) " + si.getFNumber(line));
						} else {
							//    ,    
							firstNumber = si.getFNumber(line);
							first = si;
						}
						
						last = si;
						lastNumber = si.getSNumber(line);  
					}
					
					b = true;
					//    
					this.mySet.addAll(set);
				}
				
				// .   
				marks.add(line);
				
				//   
				if (line.getSPoint().getLine().get(0).equals(line)){
					line = line.getSPoint().getLine().get(1);
				} else {
					line = line.getSPoint().getLine().get(0);
				}
			}
			
			//    
			if (!b){
				this.myRingsCount++;
			}
		}
		
	}
	
	/**
	 *   
	 */
	public Polynom getKaufman(){
		if (this.hasChanges){
			this.build();
			this.hasChanges = false;
		}
		
		if ((this.myKaufman == null) || (this.count != 0)){
			//System.out.println("Kaufman ");
			Thread t = new Thread(new Runnable(){
				public void run(){
					count++;
					if (SimpleKnot.this.mySet.size() == 0){
						int degree = SimpleKnot.this.myRingsCount + SimpleKnot.this.myTmpRingsCount;
						Polynom r = Polynom.createMonom(2);
						r.add(Polynom.createMonom(-2));
						r.mult(0,-1);
						r.degree(degree-1);
						//System.out.println("size = 0 r = " + r + " degree = " + degree);
						count--;
						SimpleKnot.this.myKaufman = r;
						return;
					}
				
					SimpleIntersection si = SimpleKnot.this.mySet.iterator().next();
					SimpleKnot.this.mySet.remove(si);
					int tmp = SimpleKnot.this.myTmpRingsCount;
				
					Polynom r = Polynom.createMonom(1);
					SimpleKnot.this.myTmpRingsCount += si.connectAType();
					//System.out.println(count+" THIS A type "+ this);
					r.mult(SimpleKnot.this.getKaufman());
					//System.out.println(count+" A type " + si.myName +" " + this.mySet.size() + " r = " + r);
					//si.setAsWas();
					SimpleKnot.this.myTmpRingsCount = tmp;
				
					//System.out.println(count+"	 "+ this);
					SimpleKnot.this.myTmpRingsCount += si.connectBType();
					//System.out.println(count+" THIS B type "+ this);
					Polynom p = Polynom.createMonom(-1).mult(SimpleKnot.this.getKaufman());
					//System.out.println(count+" p " + si.myName +" " + p);
					r.add(p);
					//System.out.println(count+" B type " + si.myName +" " + this.mySet.size() + " r = " + r);
					si.setAsWas();
				
					SimpleKnot.this.myTmpRingsCount = 0;
					SimpleKnot.this.mySet.add(si);
				
					count--;
					SimpleKnot.this.myKaufman = r;
				}
				
			});
			t.run();
			
			return this.myKaufman;
		} else {
			return this.myKaufman;
		}
	}
	
	/**
	 *   -
	 */
	public Polynom getJonesKaufman(){
		if (this.hasChanges){
			this.build();
			this.hasChanges = false;
		}
		if (this.myJonesKaufman == null){
			//System.out.println("JonesKaufman ");
			int w = 0;
			for (SimpleIntersection si: this.mySet){
				w += si.getSignum();
			}
		
			w = -3*w;
		
			Polynom r = Polynom.createMonom(w);
			r.mult(0, (int)Math.round(Math.pow(-1, w)));
			r.mult(this.getKaufman());
			this.myJonesKaufman = r;
		}
		return this.myJonesKaufman;
	}
	
	public DoublePolynom getJones(){
		if (this.hasChanges){
			this.build();
			this.hasChanges = false;
		}
		if (this.myJones == null){
			this.myJones = new DoublePolynom(this.getJonesKaufman());
			this.myJones.substitute(-0.25);
		}
		return this.myJones;
	}
	
	/**
	 *     html 
	 */
	public String getJonesInHTML(){
		if (this.hasChanges){
			this.build();
			this.hasChanges = false;
		}
		StringBuffer sb = new StringBuffer();
		//sb.append("<html>");
		Set<Integer> set = this.getJonesKaufman().degreeSet();
		//System.out.println(set);
		if (set.size() == 0){
			return "";
		}
		
		Integer min = set.iterator().next();
		
		for (Integer i: set){
			if (min != i){
				if ((this.myJonesKaufman.getCoefficient(i) >= 0)){
					sb.append(" + ");
				} else {
					sb.append(" - ");
				}
			} else if (this.myJonesKaufman.getCoefficient(i) < 0){
				sb.append("- ");
			}
			if (Math.abs(this.myJonesKaufman.getCoefficient(i)) != 1){
				sb.append(Math.abs(this.myJonesKaufman.getCoefficient(i)));
			}
			if (i == 0){
				if (this.myJonesKaufman.getCoefficient(i) == 1){
					sb.append("1");
				}
			} else if (4*(i/4) == i){
				//sb.append(" ");
				sb.append("x<sup>");
				sb.append((-i/4));
				sb.append("</sup>");
			} else if (2*(i/2) == i){
				//sb.append(" ");
				sb.append("x<sup>");
				sb.append((-i/2));
				sb.append("/2</sup>");
			} else {
				//sb.append(" ");
				sb.append("x<sup>");
				sb.append((-i));
				sb.append("/4</sup>");
			}
		}
		
		//sb.append("</html>");
		return sb.toString();
	}
	
	/**
	 *  ,    
	 */
	private class IntersectionComparator implements Comparator<SimpleIntersection>{

		/**
		 *    
		 */
		private ILine getCommonLine(SimpleIntersection o1, SimpleIntersection o2){
			if(o1.getParent().getUpLine().equals(o2.getParent().getUpLine()) 
						|| o1.getParent().getUpLine().equals(o2.getParent().getDownLine())){
				return o1.getParent().getUpLine();
			} else if (o1.getParent().getDownLine().equals(o2.getParent().getUpLine()) 
						|| o1.getParent().getDownLine().equals(o2.getParent().getDownLine())){
				return o1.getParent().getDownLine();
			} else {
				throw new RuntimeException("I cant compare "+o1 + " with " + o2 + " !");
			}
		}
		
		/**
		 *  
		 */
		public int compare(SimpleIntersection o1, SimpleIntersection o2) {
			ILine line = this.getCommonLine(o1,o2);
			int fx = line.getFPoint().getX();
			int sx = line.getSPoint().getX();
			int fy = line.getFPoint().getY();
			int sy = line.getSPoint().getY();
			
			double ix1 = o1.getParent().getPosition()[0];
			double iy1 = o1.getParent().getPosition()[1];
			double ix2 = o2.getParent().getPosition()[0];
			double iy2 = o2.getParent().getPosition()[1];
			
			if (fx != sx){
				//System.err.println(o1+" "+o2);
				//System.err.println(fx+" "+sx+" "+ix1+" "+ix2);
				//System.err.println(-(fx-sx)*(int)Math.round(Math.signum(ix1-ix2)));
				return -(fx-sx)*(int)Math.round(Math.signum(ix1-ix2));
			} else {
				return -(fy-sy)*(int)Math.round(Math.signum(iy1-iy2));
			}
		}
		
	}
	
	public String toString(){
		StringBuffer sb = new StringBuffer();
		for (SimpleIntersection si: this.mySet){
			sb.append(si + "\n");
		}
		
		sb.append(this.myRingsCount);
		
		return sb.toString();
	}

	public void addChangeListener(ChangeListener listener) {
		this.myChangeListeners.add(listener);
	}
	
	private void sendChangeEvent(Object obj){
		ChangeEvent e = new ChangeEvent(obj);
		for (ChangeListener chl: this.myChangeListeners){
			chl.stateChanged(e);
		}
	}

}
