package org.beliaj.knots.diagram;

import java.util.List;
import org.beliaj.knots.diagram.impl.*;

import junit.framework.AssertionFailedError;
import junit.framework.TestCase;

public class DiagramTestCase extends TestCase {
	
	/**
	 *  double   
	 * @param msg
	 * @param req
	 * @param actual
	 * @param epsilon
	 */
	private void compareDouble(String msg, double req, double actual, double epsilon){
		if ( (req + Math.abs(epsilon) <= actual) || (req - Math.abs(epsilon) >= actual)){
			throw new AssertionFailedError(msg +" ("+req+" + " + Math.abs(epsilon)+" <= "+actual+" )" +
					" || ( "+req+" - " + Math.abs(epsilon)+ " >= " + actual + ")");
		}
	}
	
	/**
	 *    .
	 */
	public void test1(){
		
		IKnotDiagram knotDiagram = new KnotDiagram();
		
		List<IPoint> list = knotDiagram.getPoints();
		
		knotDiagram.addPoint(1, 8);
		knotDiagram.addPoint(6, 10);
		knotDiagram.addPoint(8, 5);
		
		knotDiagram.connectPoints(list.get(0), list.get(1));
		knotDiagram.connectPoints(list.get(1), list.get(2));
		
		TestCase.assertEquals("test1_1: ", "Points:" + '\n' +
				"P0: ( 1, 8 )" + '\n' + "P1: ( 6, 10 )" + '\n' + 
				"P2: ( 8, 5 )" + '\n' + "Lines:" + '\n' + 
				"L0: ( P1, P0 )" + '\n' + "L1: ( P2, P1 )\n" +
				"Intersections:\n", knotDiagram.toString()); 
		
		knotDiagram.addPoint(6, 1);
		knotDiagram.addPoint(8, 12);
		knotDiagram.addPoint(11, 10);
		
		knotDiagram.connectPoints(list.get(4), list.get(5));
		
		TestCase.assertEquals("test1_2: ", "Points:\nP0: ( 1, 8 )\n" + 
				"P1: ( 6, 10 )\n" +
				"P2: ( 8, 5 )\n" +
				"P3: ( 6, 1 )\n" +
				"P4: ( 8, 12 )\n" +
				"P5: ( 11, 10 )\n" +
				"Lines:\n" +
				"L0: ( P1, P0 )\n" +
				"L1: ( P2, P1 )\n" +
				"L2: ( P5, P4 )\n" +
				"Intersections:\n", knotDiagram.toString());
		
		knotDiagram.addPoint(10, 6);
		knotDiagram.addPoint(4, 5);
		knotDiagram.addPoint(11, 12);
		
		knotDiagram.connectPoints(list.get(5), list.get(6));
		
		TestCase.assertEquals("test1_3: ", "Points:\n" +
				"P0: ( 1, 8 )\n" +
				"P1: ( 6, 10 )\n" +
				"P2: ( 8, 5 )\n" +
				"P3: ( 6, 1 )\n" +
				"P4: ( 8, 12 )\n" +
				"P5: ( 11, 10 )\n" +
				"P6: ( 10, 6 )\n" +
				"P7: ( 4, 5 )\n" +
				"P8: ( 11, 12 )\n" +
				"Lines:\n" +
				"L0: ( P1, P0 )\n" +
				"L1: ( P2, P1 )\n" +
				"L2: ( P5, P4 )\n" +
				"L3: ( P6, P5 )\n" +
				"Intersections:\n", knotDiagram.toString());
		
		knotDiagram.connectPoints(list.get(7), list.get(8));
		
		TestCase.assertEquals("test1_4: ", "Points:\n" +
				"P0: ( 1, 8 )\n" +
				"P1: ( 6, 10 )\n" +
				"P2: ( 8, 5 )\n" +
				"P3: ( 6, 1 )\n" +
				"P4: ( 8, 12 )\n" +
				"P5: ( 11, 10 )\n" +
				"P6: ( 10, 6 )\n" +
				"P7: ( 4, 5 )\n" +
				"P8: ( 11, 12 )\n" +
				"Lines:\n" +
				"L0: ( P1, P0 )\n" +
				"L1: ( P2, P1 )\n" +
				"    I0 \n" +
				"L2: ( P5, P4 )\n" +
				"    I1 \n" +
				"L3: ( P6, P5 )\n" +
				"L4: ( P8, P7 )\n" +
				"    I0 I1 \n" +
				"Intersections:\n" +
				"I0 ( L1, L4 )\n" +
				"I1 ( L2, L4 )\n", knotDiagram.toString());
		
		knotDiagram.connectPoints(list.get(6), list.get(7));
		
		TestCase.assertEquals("test1_5: ", "Points:\n" +
				"P0: ( 1, 8 )\n" +
				"P1: ( 6, 10 )\n" +
				"P2: ( 8, 5 )\n" +
				"P3: ( 6, 1 )\n" +
				"P4: ( 8, 12 )\n" +
				"P5: ( 11, 10 )\n" +
				"P6: ( 10, 6 )\n" +
				"P7: ( 4, 5 )\n" +
				"P8: ( 11, 12 )\n" +
				"Lines:\n" +
				"L0: ( P1, P0 )\n" +
				"L1: ( P2, P1 )\n" +
				"    I0 I2 \n" +
				"L2: ( P5, P4 )\n" +
				"    I1 \n" +
				"L3: ( P6, P5 )\n" +
				"L4: ( P8, P7 )\n" +
				"    I0 I1 \n" +
				"L5: ( P7, P6 )\n" +
				"    I2 \n" +
				"Intersections:\n" +
				"I0 ( L1, L4 )\n" +
				"I2 ( L1, L5 )\n" +
				"I1 ( L2, L4 )\n", knotDiagram.toString());
		
		knotDiagram.connectPoints(list.get(2), list.get(3));
		
		TestCase.assertEquals("test1_6: ", "Points:\n" +
				"P0: ( 1, 8 )\n" +
				"P1: ( 6, 10 )\n" +
				"P2: ( 8, 5 )\n" +
				"P3: ( 6, 1 )\n" +
				"P4: ( 8, 12 )\n" +
				"P5: ( 11, 10 )\n" +
				"P6: ( 10, 6 )\n" +
				"P7: ( 4, 5 )\n" +
				"P8: ( 11, 12 )\n" +
				"Lines:\n" +
				"L0: ( P1, P0 )\n" +
				"L1: ( P2, P1 )\n" +
				"    I0 I2 \n" +
				"L2: ( P5, P4 )\n" +
				"    I1 \n" +
				"L3: ( P6, P5 )\n" +
				"L4: ( P8, P7 )\n" +
				"    I0 I1 \n" +
				"L5: ( P7, P6 )\n" +
				"    I2 \n" +
				"L6: ( P3, P2 )\n" +
				"Intersections:\n" +
				"I0 ( L1, L4 )\n" +
				"I2 ( L1, L5 )\n" +
				"I1 ( L2, L4 )\n", knotDiagram.toString());
		
		TestCase.assertTrue("test1_7",knotDiagram.removePoint(list.get(0)));
		TestCase.assertTrue("test1_8",knotDiagram.removePoint(list.get(5)));
		
		TestCase.assertEquals("test1_9: ", "Points:\n" +
				"P1: ( 6, 10 )\n" +
				"P2: ( 8, 5 )\n" +
				"P3: ( 6, 1 )\n" +
				"P4: ( 8, 12 )\n" +
				"P5: ( 11, 10 )\n" +
				"P7: ( 4, 5 )\n" +
				"P8: ( 11, 12 )\n" +
				"Lines:\n" +
				"L1: ( P2, P1 )\n" +
				"    I0 \n" +
				"L2: ( P5, P4 )\n" +
				"    I1 \n" +
				"L4: ( P8, P7 )\n" +
				"    I0 I1 \n" +
				"L6: ( P3, P2 )\n" +
				"Intersections:\n" +
				"I0 ( L1, L4 )\n" +
				"I1 ( L2, L4 )\n", knotDiagram.toString());
		
		TestCase.assertTrue("test1_10",knotDiagram.removeLine(knotDiagram.getLines().get(0)));
		
		TestCase.assertEquals("test1_11: ", "Points:\n" +
				"P1: ( 6, 10 )\n" +
				"P2: ( 8, 5 )\n" +
				"P3: ( 6, 1 )\n" +
				"P4: ( 8, 12 )\n" +
				"P5: ( 11, 10 )\n" +
				"P7: ( 4, 5 )\n" +
				"P8: ( 11, 12 )\n" +
				"Lines:\n" +
				"L2: ( P5, P4 )\n" +
				"    I1 \n" +
				"L4: ( P8, P7 )\n" +
				"    I1 \n" +
				"L6: ( P3, P2 )\n" +
				"Intersections:\n" +
				"I1 ( L2, L4 )\n", knotDiagram.toString());
		
		TestCase.assertTrue("test1_12",knotDiagram.removeLine(knotDiagram.getLines().get(2)));
		
		TestCase.assertEquals("test1_13: ", "Points:\n" +
				"P1: ( 6, 10 )\n" +
				"P2: ( 8, 5 )\n" +
				"P3: ( 6, 1 )\n" +
				"P4: ( 8, 12 )\n" +
				"P5: ( 11, 10 )\n" +
				"P7: ( 4, 5 )\n" +
				"P8: ( 11, 12 )\n" +
				"Lines:\n" +
				"L2: ( P5, P4 )\n" +
				"    I1 \n" +
				"L4: ( P8, P7 )\n" +
				"    I1 \n" +
				"Intersections:\n" +
				"I1 ( L2, L4 )\n", knotDiagram.toString());
		
	}
	
	/**
	 *  .
	 */
	public void test2(){
		KnotDiagram knotDiagram = new KnotDiagram();
		
		List<IPoint> list = knotDiagram.getPoints();
		
		knotDiagram.addPoint(2, 6);
		knotDiagram.addPoint(8, 6);
		knotDiagram.addPoint(6, 9);
		knotDiagram.addPoint(6, 4);
		
		TestCase.assertEquals("10: ", "Points:\n" +
				"P0: ( 2, 6 )\n" +
				"P1: ( 8, 6 )\n" +
				"P2: ( 6, 9 )\n" +
				"P3: ( 6, 4 )\n" +
				"Lines:\n" +
				"Intersections:\n", knotDiagram.toString());
		
		knotDiagram.connectPoints(list.get(0), list.get(1));
		knotDiagram.connectPoints(list.get(2), list.get(3));
		
		TestCase.assertEquals("11: ", "Points:\n" +
				"P0: ( 2, 6 )\n" +
				"P1: ( 8, 6 )\n" +
				"P2: ( 6, 9 )\n" +
				"P3: ( 6, 4 )\n" +
				"Lines:\n" +
				"L0: ( P1, P0 )\n" +
				"    I0 \n" +
				"L1: ( P3, P2 )\n" +
				"    I0 \n" +
				"Intersections:\n" +
				"I0 ( L0, L1 )\n", knotDiagram.toString());
		
		knotDiagram.connectPoints(list.get(2), list.get(0));
		
		TestCase.assertEquals("12: ", "Points:\n" +
				"P0: ( 2, 6 )\n" +
				"P1: ( 8, 6 )\n" +
				"P2: ( 6, 9 )\n" +
				"P3: ( 6, 4 )\n" +
				"Lines:\n" +
				"L0: ( P0, P1 )\n" +
				"    I0 \n" +
				"L1: ( P3, P2 )\n" +
				"    I0 \n" +
				"L2: ( P2, P0 )\n" +
				"Intersections:\n" +
				"I0 ( L0, L1 )\n", knotDiagram.toString());
		
	}
	
	/**
	 *     .
	 */
	public void test3(){
		KnotDiagram knotDiagram = new KnotDiagram();
		List<IPoint> list = knotDiagram.getPoints();
		List<ILine> lines = knotDiagram.getLines();
		
		knotDiagram.addPoint(2, 1);
		knotDiagram.addPoint(9, 7);
		knotDiagram.addPoint(2, 7);
		knotDiagram.addPoint(9, 1);
		
		//System.out.println(knotDiagram);
		TestCase.assertEquals("test3_1: ", "Points:\n" +
				"P0: ( 2, 1 )\n" +
				"P1: ( 9, 7 )\n" +
				"P2: ( 2, 7 )\n" +
				"P3: ( 9, 1 )\n" +
				"Lines:\n" +
				"Intersections:\n", knotDiagram.toString());
		
		knotDiagram.connectPoints(list.get(1),list.get(2));
		knotDiagram.connectPoints(list.get(1),list.get(3));
		knotDiagram.connectPoints(list.get(0),list.get(2));
		knotDiagram.connectPoints(list.get(0),list.get(3));
		
		//System.out.println(knotDiagram);
		TestCase.assertEquals("test3_2: ", "Points:\n" +
				"P0: ( 2, 1 )\n" +
				"P1: ( 9, 7 )\n" +
				"P2: ( 2, 7 )\n" +
				"P3: ( 9, 1 )\n" +
				"Lines:\n" +
				"L0: ( P2, P1 )\n" +
				"L1: ( P1, P3 )\n" +
				"L2: ( P0, P2 )\n" +
				"L3: ( P3, P0 )\n" +
				"Intersections:\n", knotDiagram.toString());
		
		TestCase.assertEquals("test3_3: Fit: (6,4,4) ",
				lines.get(0), 
				knotDiagram.findLine(6,4,4));

		TestCase.assertEquals("test3_4: Fit: (4,6,4) ",
				lines.get(0), 
				knotDiagram.findLine(4,6,4));
		
		TestCase.assertNull("test3_5: Fit: (11,8,10) ", 
				knotDiagram.findLine(11,8,10));
		
		TestCase.assertEquals("test3_6: Fit: (0,4,10) ",
				lines.get(2), 
				knotDiagram.findLine(0,4,10));
		
		TestCase.assertNull("test3_7: Fit: (0,4,1) ",
				knotDiagram.findLine(0,4,1));
	}
	
	/**
	 *     .
	 */
	public void test4(){
		IKnotDiagram knotDiagram = new KnotDiagram();
		List<IPoint> list = knotDiagram.getPoints();
		
		knotDiagram.addPoint(2, 3);
		knotDiagram.addPoint(6, 1);
		knotDiagram.addPoint(3, 1);
		
		//System.out.println(knotDiagram);
		
		TestCase.assertEquals("test4_1: ", "Points:\n" +
				"P0: ( 2, 3 )\n" +
				"P1: ( 6, 1 )\n" +
				"P2: ( 3, 1 )\n" +
				"Lines:\n" +
				"Intersections:\n", knotDiagram.toString());
		
		//System.out.println("test4_2: FirstFit: (3, 2, 2) " + knotDiagram.findPointFirstFit(3, 2, 2));
		TestCase.assertEquals("test4_2: FirstFit: (3, 2, 2) ",
				list.get(2),
				knotDiagram.findPoint(3, 2, 2));
		
		//System.out.println("test4_3: FirstFit: (5, 2, 3) " + knotDiagram.findPointFirstFit(5, 2, 3));
		TestCase.assertEquals("test4_3: FirstFit: (5, 2, 3) ",
				list.get(1),
				knotDiagram.findPoint(5, 2, 3));
		
		//System.out.println("test4_4: FirstFit: (6, 7, 1) " + knotDiagram.findPointFirstFit(6, 7, 1));
		TestCase.assertNull("test4_4: FirstFit: (6, 7, 1) ",
				knotDiagram.findPoint(6, 7, 1));
		
		TestCase.assertNull("test4_5: FirstFit: (5, 2, 1) ",
				knotDiagram.findPoint(5, 2, 1));
		
	}
	
	/**
	 *     .
	 *
	 */
	public void test5(){
		IKnotDiagram knotDiagram = new KnotDiagram();
		List<IPoint> list = knotDiagram.getPoints();
		
		knotDiagram.addPoint(-3, 4);
		knotDiagram.addPoint(6, 4);
		knotDiagram.addPoint(-2, 2);
		knotDiagram.addPoint(3, 2);
		knotDiagram.addPoint(-1, -1);
		knotDiagram.addPoint(-1, 1);
		
		//System.out.println(knotDiagram);
		TestCase.assertEquals("test5_1: ",
				"Points:\n" +
				"P0: ( -3, 4 )\n" +
				"P1: ( 6, 4 )\n" +
				"P2: ( -2, 2 )\n" +
				"P3: ( 3, 2 )\n" +
				"P4: ( -1, -1 )\n" +
				"P5: ( -1, 1 )\n" +
				"Lines:\n" +
				"Intersections:\n",
				knotDiagram.toString());
		
		knotDiagram.connectPoints(list.get(0), list.get(1));
		knotDiagram.connectPoints(list.get(2), list.get(3));
		knotDiagram.connectPoints(list.get(4), list.get(5));
		
		//System.out.println(knotDiagram);
		TestCase.assertEquals("test5_2: ",
				"Points:\n" +
				"P0: ( -3, 4 )\n" +
				"P1: ( 6, 4 )\n" +
				"P2: ( -2, 2 )\n" +
				"P3: ( 3, 2 )\n" +
				"P4: ( -1, -1 )\n" +
				"P5: ( -1, 1 )\n" +
				"Lines:\n" +
				"L0: ( P1, P0 )\n" +
				"L1: ( P3, P2 )\n" +
				"L2: ( P5, P4 )\n" +
				"Intersections:\n",
				knotDiagram.toString());
		
		//System.out.println(knotDiagram.getMaxCoordinate());
		TestCase.assertEquals("test5_3: ", 6, Math.max(knotDiagram.getMaxXCoordinate(),
				knotDiagram.getMaxYCoordinate()));
		
		//System.out.println(knotDiagram.getMinCoordinate());
		TestCase.assertEquals("test5_4: ", -3, Math.min(knotDiagram.getMinXCoordinate(),
				knotDiagram.getMinYCoordinate()));
		
		TestCase.assertEquals("test5_5: ", -3, knotDiagram.getMinXCoordinate());
		TestCase.assertEquals("test5_6: ", -1, knotDiagram.getMinYCoordinate());
		
		TestCase.assertEquals("test5_7: ", 6, knotDiagram.getMaxXCoordinate());
		TestCase.assertEquals("test5_8: ", 4, knotDiagram.getMaxYCoordinate());
		
	}
	
	/**
	 *     .
	 *
	 */
	public void test6(){
		IKnotDiagram knotDiagram = new KnotDiagram();
		List<IPoint> list = knotDiagram.getPoints();
		
		knotDiagram.addPoint(-4, 4);
		knotDiagram.addPoint(-4, 0);
		knotDiagram.addPoint(-2, 6);
		knotDiagram.addPoint(-2, -1);
		knotDiagram.addPoint(4, 4);
		knotDiagram.addPoint(5, 6);
		
		//System.out.println(knotDiagram);
		TestCase.assertEquals("test6_1: ", "Points:\n" +
				"P0: ( -4, 4 )\n" +
				"P1: ( -4, 0 )\n" +
				"P2: ( -2, 6 )\n" +
				"P3: ( -2, -1 )\n" +
				"P4: ( 4, 4 )\n" +
				"P5: ( 5, 6 )\n" +
				"Lines:\n" +
				"Intersections:\n", knotDiagram.toString());
		
		knotDiagram.connectPoints(list.get(0), list.get(4));
		knotDiagram.connectPoints(list.get(1), list.get(5));
		knotDiagram.connectPoints(list.get(2), list.get(3));
		
		//System.out.println(knotDiagram);
		TestCase.assertEquals("test6_2: ", "Points:\n" +
				"P0: ( -4, 4 )\n" +
				"P1: ( -4, 0 )\n" +
				"P2: ( -2, 6 )\n" +
				"P3: ( -2, -1 )\n" +
				"P4: ( 4, 4 )\n" +
				"P5: ( 5, 6 )\n" +
				"Lines:\n" +
				"L0: ( P4, P0 )\n" +
				"    I0 I1 \n" +
				"L1: ( P5, P1 )\n" +
				"    I0 I2 \n" +
				"L2: ( P3, P2 )\n" +
				"    I1 I2 \n" +
				"Intersections:\n" +
				"I0 ( L0, L1 )\n" +
				"I1 ( L0, L2 )\n" +
				"I2 ( L1, L2 )\n", knotDiagram.toString());
		
		List<ILine> lines = knotDiagram.getLines();
		
		
		this.compareDouble("test6_3: ",
				1.9999999999999996,
				lines.get(0).getIntersections().get(0).getPosition()[0],
				0.0000000000000005);
		
		this.compareDouble("test6_4: ",
				4.0,
				lines.get(0).getIntersections().get(0).getPosition()[1],
				0.0000000000000005);
		
		this.compareDouble("test6_5: ",
				-2.0, lines.get(0).getIntersections().get(1).getPosition()[0],
				0.0000000000000005);
		
		this.compareDouble("test6_6: ",
				4.0, lines.get(0).getIntersections().get(1).getPosition()[1],
				0.0000000000000005);
		
		this.compareDouble("test6_7: ",
				-2.0, lines.get(1).getIntersections().get(1).getPosition()[0],
				0.0000000000000005);
		
		this.compareDouble("test6_8: ",
				1.3333333333333335, lines.get(1).getIntersections().get(1).getPosition()[1],
				0.0000000000000005);
		
	}
	
	/**
	 *   canConnect
	 *
	 */
	public void test7(){
		IKnotDiagram knotDiagram = new KnotDiagram();
		List<IPoint> list = knotDiagram.getPoints();
		
		knotDiagram.addPoint(-1, 5);
		knotDiagram.addPoint(5, 6);
		knotDiagram.addPoint(1, 0);
		knotDiagram.addPoint(1, 8);
		knotDiagram.addPoint(4, 1);
		
		//System.out.println(knotDiagram);
		TestCase.assertEquals("test7_1: ", "Points:\n" +
				"P0: ( -1, 5 )\n" +
				"P1: ( 5, 6 )\n" +
				"P2: ( 1, 0 )\n" +
				"P3: ( 1, 8 )\n" +
				"P4: ( 4, 1 )\n" +
				"Lines:\n" +
				"Intersections:\n", knotDiagram.toString());
		
		knotDiagram.connectPoints(list.get(0), list.get(1));
		knotDiagram.connectPoints(list.get(1), list.get(2));
		knotDiagram.connectPoints(list.get(4), list.get(3));
		
		//System.out.println(knotDiagram);
		TestCase.assertEquals("test7_2: ", "Points:\n" +
				"P0: ( -1, 5 )\n" +
				"P1: ( 5, 6 )\n" +
				"P2: ( 1, 0 )\n" +
				"P3: ( 1, 8 )\n" +
				"P4: ( 4, 1 )\n" +
				"Lines:\n" +
				"L0: ( P1, P0 )\n" +
				"    I0 \n" +
				"L1: ( P2, P1 )\n" +
				"    I1 \n" +
				"L2: ( P3, P4 )\n" +
				"    I0 I1 \n" +
				"Intersections:\n" +
				"I0 ( L0, L2 )\n" +
				"I1 ( L1, L2 )\n", knotDiagram.toString());
		
		try{
			knotDiagram.connectPoints(list.get(4), list.get(1));
		} catch (LineException le){
			TestCase.assertEquals("test7_3: ", "Points:\n" +
					"P0: ( -1, 5 )\n" +
					"P1: ( 5, 6 )\n" +
					"P2: ( 1, 0 )\n" +
					"P3: ( 1, 8 )\n" +
					"P4: ( 4, 1 )\n" +
					"Lines:\n" +
					"L0: ( P1, P0 )\n" +
					"    I0 \n" +
					"L1: ( P2, P1 )\n" +
					"    I1 \n" +
					"L2: ( P3, P4 )\n" +
					"    I0 I1 \n" +
					"Intersections:\n" +
					"I0 ( L0, L2 )\n" +
					"I1 ( L1, L2 )\n", knotDiagram.toString());
			TestCase.assertEquals("test7_4: ", false, list.get(4).canConnect() && list.get(1).canConnect());
		}
		
		if (list.get(0).canConnect() && list.get(2).canConnect()){
			knotDiagram.connectPoints(list.get(2), list.get(0));
		}
		
		try{
			knotDiagram.connectPoints(list.get(0), list.get(3));
		} catch (LineException le){
			//System.out.println(knotDiagram);
			TestCase.assertEquals("test7_5: ", "Points:\n" +
					"P0: ( -1, 5 )\n" +
					"P1: ( 5, 6 )\n" +
					"P2: ( 1, 0 )\n" +
					"P3: ( 1, 8 )\n" +
					"P4: ( 4, 1 )\n" +
					"Lines:\n" +
					"L0: ( P1, P0 )\n" +
					"    I0 \n" +
					"L1: ( P2, P1 )\n" +
					"    I1 \n" +
					"L2: ( P3, P4 )\n" +
					"    I0 I1 \n" +
					"L3: ( P0, P2 )\n" +
					"Intersections:\n" +
					"I0 ( L0, L2 )\n" +
					"I1 ( L1, L2 )\n", knotDiagram.toString());
			TestCase.assertEquals("test7_6: ", false, list.get(0).canConnect() && list.get(3).canConnect());
		}
	}
	
	/**
	 *   
	 *
	 */
	public void test8(){
		IKnotDiagram kD1 = new KnotDiagram();
		List<IPoint> list1 = kD1.getPoints();
		
		kD1.addPoint(1,1);
		kD1.addPoint(10,10);
		
		IKnotDiagram kD2 = new KnotDiagram();
		List<IPoint> list2 = kD2.getPoints();

		kD2.addPoint(1,1);
		kD2.addPoint(10,10);
		
		RuntimeException re1 = null;
		RuntimeException re2 = null;
		
		try{
			kD1.connectPoints(list1.get(0), list2.get(0));
		} catch (MixedDiagramsException mde){
			re1 = mde;
		}
		
		try{
			kD2.connectPoints(list1.get(1), list2.get(1));
		} catch (MixedDiagramsException mde){
			re2 = mde;
		}
		
		TestCase.assertNotNull("test8_1",re1);
		TestCase.assertNotNull("test8_2",re2);
		re1 = null;
		re2 = null;
		
		kD1.connectPoints(list1.get(1), list1.get(0));
		kD2.connectPoints(list2.get(1), list2.get(0));
		
		//System.out.println(kD1);
		TestCase.assertEquals("test8_2,5", "Points:\n" +
				"P0: ( 1, 1 )\n" +
				"P1: ( 10, 10 )\n" +
				"Lines:\n" +
				"L0: ( P0, P1 )\n" +
				"Intersections:\n", kD1.toString());
		//System.out.println(kD2);
		TestCase.assertEquals("test8_2,5", "Points:\n" +
				"P0: ( 1, 1 )\n" +
				"P1: ( 10, 10 )\n" +
				"Lines:\n" +
				"L0: ( P0, P1 )\n" +
				"Intersections:\n", kD2.toString());
		
		List<ILine> lines1 = kD1.getLines();
		List<ILine> lines2 = kD2.getLines();
		
		try{
			TestCase.assertFalse("test8_2,3",kD1.removeLine(lines2.get(0)));
		} catch (RuntimeException mde){
			re1 = mde;
		}
		
		try{
			TestCase.assertFalse("test8_2,6",kD2.removeLine(lines1.get(0)));
		} catch (RuntimeException mde){
			re2 = mde;
		}
		
		TestCase.assertNull("test8_3",re1);
		TestCase.assertNull("test8_4",re2);
		re1 = null;
		re2 = null;
		
		try{
			TestCase.assertFalse("test8_4,3",kD1.removePoint(list2.get(0)));
		} catch (RuntimeException mde){
			re1 = mde;
		}
		
		try{
			TestCase.assertFalse("test8_4,6",kD2.removePoint(list1.get(0)));
		} catch (RuntimeException mde){
			re2 = mde;
		}
		
		TestCase.assertNull("test8_5",re1);
		TestCase.assertNull("test8_6",re2);
		re1 = null;
		re2 = null;
		
		try{
			kD1.movePoint(list2.get(1), 2,2);
		} catch (MixedDiagramsException mde){
			re1 = mde;
		}
		
		try{
			kD2.movePoint(list1.get(1), 2,2);
		} catch (MixedDiagramsException mde){
			re2 = mde;
		}
		
		TestCase.assertNotNull("test8_7",re1);
		TestCase.assertNotNull("test8_8",re2);
		re1 = null;
		re2 = null;
		
		try{
			kD2.changeLineDirection(lines1.get(0));
		} catch (MixedDiagramsException mde){
			re2 = mde;
		}
		
		TestCase.assertNotNull("test8_9",re2);
		
		try{
			kD1.changeLineDirection(lines2.get(0));
		} catch (MixedDiagramsException mde){
			re2 = mde;
		}
		
		TestCase.assertNotNull("test8_10",re2);
		
	}
	
	/**
	 *   null
	 *
	 */
	public void test9(){
		IKnotDiagram kD1 = new KnotDiagram();
		List<IPoint> list1 = kD1.getPoints();
		
		kD1.addPoint(1,1);
		kD1.addPoint(10,10);
		
		RuntimeException re1 = null;
		
		try{
			kD1.connectPoints(list1.get(0), null);
		} catch (MixedDiagramsException mde){
			re1 = mde;
		}
		
		TestCase.assertNotNull("test9_1",re1);
		re1 = null;
		
		kD1.connectPoints(list1.get(1), list1.get(0));
		
		TestCase.assertFalse("test9_2",kD1.removeLine(null));
		
		TestCase.assertFalse("test9_3",kD1.removePoint(null));
		
		try{
			kD1.movePoint(null, 2,2);
		} catch (MixedDiagramsException mde){
			re1 = mde;
		}
		
		TestCase.assertNotNull("test9_4",re1);
		re1 = null;
		
		TestCase.assertEquals("test9_5", "Points:\n" +
				"P0: ( 1, 1 )\n" +
				"P1: ( 10, 10 )\n" +
				"Lines:\n" +
				"L0: ( P0, P1 )\n" +
				"Intersections:\n", kD1.toString());
		
		try{
			kD1.changeLineDirection(null);
		} catch (NullPointerException e){
			re1 = e;
		}
		
		TestCase.assertNotNull("test9_6",re1);
		re1 = null;
		
	}
	
	/**
	 *   swapLines
	 *
	 */
	public void test10(){
		IKnotDiagram knotDiagram = new KnotDiagram();
		List<IPoint> list = knotDiagram.getPoints();
		
		knotDiagram.addPoint(1, 1);
		knotDiagram.addPoint(4, 4);
		knotDiagram.addPoint(4, 1);
		knotDiagram.addPoint(1, 4);
		
		knotDiagram.connectPoints(list.get(0), list.get(1));
		knotDiagram.connectPoints(list.get(2), list.get(3));
		
		knotDiagram.setIntersectionType(knotDiagram.getLines().get(0), knotDiagram.getLines().get(1));
		
		TestCase.assertEquals("test10_1", "Points:\n" +
				"P0: ( 1, 1 )\n" +
				"P1: ( 4, 4 )\n" +
				"P2: ( 4, 1 )\n" +
				"P3: ( 1, 4 )\n" +
				"Lines:\n" +
				"L0: ( P1, P0 )\n" +
				"    I0 \n" +
				"L1: ( P3, P2 )\n" +
				"    I0 \n" +
				"Intersections:\n" +
				"I0 ( L0, L1 )\n"
				, knotDiagram.toString());
		
		knotDiagram.swapLines(knotDiagram.getLines().get(0).getIntersections().get(0));
		IIntersection ii = knotDiagram.getLines().get(0).getIntersections().get(0);
		
		TestCase.assertEquals("test10_1", "Points:\n" +
				"P0: ( 1, 1 )\n" +
				"P1: ( 4, 4 )\n" +
				"P2: ( 4, 1 )\n" +
				"P3: ( 1, 4 )\n" +
				"Lines:\n" +
				"L0: ( P1, P0 )\n" +
				"    I0 \n" +
				"L1: ( P3, P2 )\n" +
				"    I0 \n" +
				"Intersections:\n" +
				"I0 ( L1, L0 )\n"
				, knotDiagram.toString());
		
		TestCase.assertFalse("test10_2",knotDiagram.swapLines(null));
		knotDiagram.removePoint(list.get(0));
		
		//System.out.println(knotDiagram);
		
		RuntimeException re1 = null;
		
		try{
			knotDiagram.swapLines(ii);
		} catch (MixedDiagramsException mde){
			re1 = mde;
		}
		
		TestCase.assertNotNull("test10_3",re1);
	}
	
	/**
	 *    
	 *
	 */
	public void test11(){
		IKnotDiagram knotDiagram = new KnotDiagram();
		
		List<IPoint> list = knotDiagram.getPoints();
		
		knotDiagram.addPoint(3, 5);
		knotDiagram.addPoint(-5, -2);
		knotDiagram.addPoint(-2, 5);
		knotDiagram.addPoint(0, 6);
		knotDiagram.addPoint(5, 1);
		knotDiagram.addPoint(6, -2);
		knotDiagram.addPoint(4, -3);
		knotDiagram.addPoint(-2, -2);
		
		knotDiagram.connectPoints(list.get(0), list.get(1));
		knotDiagram.connectPoints(list.get(1), list.get(6));
		knotDiagram.connectPoints(list.get(2), list.get(5));
		knotDiagram.connectPoints(list.get(3), list.get(7));
		knotDiagram.connectPoints(list.get(4), list.get(7));
		knotDiagram.connectPoints(list.get(3), list.get(4));
		
		//System.out.println(knotDiagram);
		
		TestCase.assertEquals("test11_1: ", "Points:\n" +
				"P0: ( 3, 5 )\n" +
				"P1: ( -5, -2 )\n" +
				"P2: ( -2, 5 )\n" +
				"P3: ( 0, 6 )\n" +
				"P4: ( 5, 1 )\n" +
				"P5: ( 6, -2 )\n" +
				"P6: ( 4, -3 )\n" +
				"P7: ( -2, -2 )\n" +
				"Lines:\n" +
				"L0: ( P1, P0 )\n" +
				"    I0 I1 I4 \n" +
				"L1: ( P6, P1 )\n" +
				"L2: ( P5, P2 )\n" +
				"    I0 I2 I3 \n" +
				"L3: ( P7, P3 )\n" +
				"    I1 I2 \n" +
				"L4: ( P4, P7 )\n" +
				"    I3 \n" +
				"L5: ( P3, P4 )\n" +
				"    I4 \n" +
				"Intersections:\n" +
				"I0 ( L0, L2 )\n" +
				"I1 ( L0, L3 )\n" +
				"I4 ( L0, L5 )\n" +
				"I2 ( L2, L3 )\n" +
				"I3 ( L2, L4 )\n", knotDiagram.toString()); 
		
		knotDiagram.setIntersectionType(knotDiagram.getLines().get(3), knotDiagram.getLines().get(2));
		knotDiagram.setIntersectionType(knotDiagram.getLines().get(4), knotDiagram.getLines().get(2));
		knotDiagram.setIntersectionType(knotDiagram.getLines().get(2), knotDiagram.getLines().get(0));
		
		TestCase.assertEquals("test11_2: ", "Points:\n" +
				"P0: ( 3, 5 )\n" +
				"P1: ( -5, -2 )\n" +
				"P2: ( -2, 5 )\n" +
				"P3: ( 0, 6 )\n" +
				"P4: ( 5, 1 )\n" +
				"P5: ( 6, -2 )\n" +
				"P6: ( 4, -3 )\n" +
				"P7: ( -2, -2 )\n" +
				"Lines:\n" +
				"L0: ( P1, P0 )\n" +
				"    I0 I1 I4 \n" +
				"L1: ( P6, P1 )\n" +
				"L2: ( P5, P2 )\n" +
				"    I0 I2 I3 \n" +
				"L3: ( P7, P3 )\n" +
				"    I1 I2 \n" +
				"L4: ( P4, P7 )\n" +
				"    I3 \n" +
				"L5: ( P3, P4 )\n" +
				"    I4 \n" +
				"Intersections:\n" +
				"I1 ( L0, L3 )\n" +
				"I4 ( L0, L5 )\n" +
				"I0 ( L2, L0 )\n" +
				"I2 ( L3, L2 )\n" +
				"I3 ( L4, L2 )\n", knotDiagram.toString());
		
		knotDiagram.resize(2.0);
		TestCase.assertEquals("test11_3: ", "Points:\n" +
				"P0: ( 6, 10 )\n" +
				"P1: ( -10, -4 )\n" +
				"P2: ( -4, 10 )\n" +
				"P3: ( 0, 12 )\n" +
				"P4: ( 10, 2 )\n" +
				"P5: ( 12, -4 )\n" +
				"P6: ( 8, -6 )\n" +
				"P7: ( -4, -4 )\n" +
				"Lines:\n" +
				"L0: ( P1, P0 )\n" +
				"    I0 I1 I4 \n" +
				"L1: ( P6, P1 )\n" +
				"L2: ( P5, P2 )\n" +
				"    I0 I2 I3 \n" +
				"L3: ( P7, P3 )\n" +
				"    I1 I2 \n" +
				"L4: ( P4, P7 )\n" +
				"    I3 \n" +
				"L5: ( P3, P4 )\n" +
				"    I4 \n" +
				"Intersections:\n" +
				"I1 ( L0, L3 )\n" +
				"I4 ( L0, L5 )\n" +
				"I0 ( L2, L0 )\n" +
				"I2 ( L3, L2 )\n" +
				"I3 ( L4, L2 )\n", knotDiagram.toString());
		
		knotDiagram.resize(5.0);
		TestCase.assertEquals("test11_4: ", "Points:\n" +
				"P0: ( 30, 50 )\n" +
				"P1: ( -50, -20 )\n" +
				"P2: ( -20, 50 )\n" +
				"P3: ( 0, 60 )\n" +
				"P4: ( 50, 10 )\n" +
				"P5: ( 60, -20 )\n" +
				"P6: ( 40, -30 )\n" +
				"P7: ( -20, -20 )\n" +
				"Lines:\n" +
				"L0: ( P1, P0 )\n" +
				"    I0 I1 I4 \n" +
				"L1: ( P6, P1 )\n" +
				"L2: ( P5, P2 )\n" +
				"    I0 I2 I3 \n" +
				"L3: ( P7, P3 )\n" +
				"    I1 I2 \n" +
				"L4: ( P4, P7 )\n" +
				"    I3 \n" +
				"L5: ( P3, P4 )\n" +
				"    I4 \n" +
				"Intersections:\n" +
				"I1 ( L0, L3 )\n" +
				"I4 ( L0, L5 )\n" +
				"I0 ( L2, L0 )\n" +
				"I2 ( L3, L2 )\n" +
				"I3 ( L4, L2 )\n", knotDiagram.toString());
		
		knotDiagram.resize(0.1);
		TestCase.assertEquals("test11_5: ", "Points:\n" +
				"P0: ( 3, 5 )\n" +
				"P1: ( -5, -2 )\n" +
				"P2: ( -2, 5 )\n" +
				"P3: ( 0, 6 )\n" +
				"P4: ( 5, 1 )\n" +
				"P5: ( 6, -2 )\n" +
				"P6: ( 4, -3 )\n" +
				"P7: ( -2, -2 )\n" +
				"Lines:\n" +
				"L0: ( P1, P0 )\n" +
				"    I0 I1 I4 \n" +
				"L1: ( P6, P1 )\n" +
				"L2: ( P5, P2 )\n" +
				"    I0 I2 I3 \n" +
				"L3: ( P7, P3 )\n" +
				"    I1 I2 \n" +
				"L4: ( P4, P7 )\n" +
				"    I3 \n" +
				"L5: ( P3, P4 )\n" +
				"    I4 \n" +
				"Intersections:\n" +
				"I1 ( L0, L3 )\n" +
				"I4 ( L0, L5 )\n" +
				"I0 ( L2, L0 )\n" +
				"I2 ( L3, L2 )\n" +
				"I3 ( L4, L2 )\n", knotDiagram.toString());
	}
	
	/**
	 * 
	 *
	 */
	public void test13(){
		IKnotDiagram knotDiagram = new KnotDiagram();
		
		List<IPoint> list = knotDiagram.getPoints();
		
		knotDiagram.addPoint(2, 2);
		knotDiagram.addPoint(3, 2);
		knotDiagram.addPoint(3, 3);
		knotDiagram.addPoint(3, 4);
		knotDiagram.addPoint(4, 4);
		knotDiagram.addPoint(4, 3);
		knotDiagram.addPoint(4, 2);
		knotDiagram.addPoint(5, 2);
		knotDiagram.addPoint(5, 4);
		knotDiagram.addPoint(5, 7);
		knotDiagram.addPoint(5, 9);
		knotDiagram.addPoint(3, 9);
		knotDiagram.addPoint(2, 9);
		knotDiagram.addPoint(3, 8);
		knotDiagram.addPoint(4, 8);
		knotDiagram.addPoint(3, 7);
		knotDiagram.addPoint(4, 7);
		knotDiagram.addPoint(2, 7);
		knotDiagram.addPoint(2, 4);
		knotDiagram.addPoint(4, 9);
		
		TestCase.assertEquals("test13_1: ", "Points:\n" +
				"P0: ( 2, 2 )\n" +
				"P1: ( 3, 2 )\n" +
				"P2: ( 3, 3 )\n" +
				"P3: ( 3, 4 )\n" +
				"P4: ( 4, 4 )\n" +
				"P5: ( 4, 3 )\n" +
				"P6: ( 4, 2 )\n" +
				"P7: ( 5, 2 )\n" +
				"P8: ( 5, 4 )\n" +
				"P9: ( 5, 7 )\n" +
				"P10: ( 5, 9 )\n" +
				"P11: ( 3, 9 )\n" +
				"P12: ( 2, 9 )\n" +
				"P13: ( 3, 8 )\n" +
				"P14: ( 4, 8 )\n" +
				"P15: ( 3, 7 )\n" +
				"P16: ( 4, 7 )\n" +
				"P17: ( 2, 7 )\n" +
				"P18: ( 2, 4 )\n" +
				"P19: ( 4, 9 )\n" +
				"Lines:\n" +
				"Intersections:\n", knotDiagram.toString());
		
		knotDiagram.connectPoints(list.get(0), list.get(1));
		knotDiagram.connectPoints(list.get(6), list.get(7));
		knotDiagram.connectPoints(list.get(12), list.get(11));
		knotDiagram.connectPoints(list.get(19), list.get(10));
		knotDiagram.connectPoints(list.get(17), list.get(4));
		knotDiagram.connectPoints(list.get(15), list.get(8));
		knotDiagram.connectPoints(list.get(16), list.get(18));
		knotDiagram.connectPoints(list.get(9), list.get(3));
		knotDiagram.connectPoints(list.get(11), list.get(14));
		knotDiagram.connectPoints(list.get(19), list.get(13));
		knotDiagram.connectPoints(list.get(2), list.get(6));
		knotDiagram.connectPoints(list.get(5), list.get(1));
		knotDiagram.connectPoints(list.get(0), list.get(18));
		knotDiagram.connectPoints(list.get(12), list.get(17));
		knotDiagram.connectPoints(list.get(13), list.get(15));
		knotDiagram.connectPoints(list.get(16), list.get(14));
		knotDiagram.connectPoints(list.get(3), list.get(2));
		knotDiagram.connectPoints(list.get(5), list.get(4));
		knotDiagram.connectPoints(list.get(8), list.get(7));
		knotDiagram.connectPoints(list.get(9), list.get(10));
		
		//System.out.println(knotDiagram);
		TestCase.assertEquals("test13_2: ", "Points:\n" +
				"P0: ( 2, 2 )\n" +
				"P1: ( 3, 2 )\n" +
				"P2: ( 3, 3 )\n" +
				"P3: ( 3, 4 )\n" +
				"P4: ( 4, 4 )\n" +
				"P5: ( 4, 3 )\n" +
				"P6: ( 4, 2 )\n" +
				"P7: ( 5, 2 )\n" +
				"P8: ( 5, 4 )\n" +
				"P9: ( 5, 7 )\n" +
				"P10: ( 5, 9 )\n" +
				"P11: ( 3, 9 )\n" +
				"P12: ( 2, 9 )\n" +
				"P13: ( 3, 8 )\n" +
				"P14: ( 4, 8 )\n" +
				"P15: ( 3, 7 )\n" +
				"P16: ( 4, 7 )\n" +
				"P17: ( 2, 7 )\n" +
				"P18: ( 2, 4 )\n" +
				"P19: ( 4, 9 )\n" +
				"Lines:\n" +
				"L0: ( P1, P0 )\n" +
				"L1: ( P7, P6 )\n" +
				"L2: ( P11, P12 )\n" +
				"L3: ( P10, P19 )\n" +
				"L4: ( P17, P4 )\n" +
				"    I0 I2 \n" +
				"L5: ( P15, P8 )\n" +
				"    I1 I3 \n" +
				"L6: ( P18, P16 )\n" +
				"    I0 I1 \n" +
				"L7: ( P3, P9 )\n" +
				"    I2 I3 \n" +
				"L8: ( P14, P11 )\n" +
				"    I4 \n" +
				"L9: ( P19, P13 )\n" +
				"    I4 \n" +
				"L10: ( P6, P2 )\n" +
				"    I5 \n" +
				"L11: ( P5, P1 )\n" +
				"    I5 \n" +
				"L12: ( P0, P18 )\n" +
				"L13: ( P12, P17 )\n" +
				"L14: ( P13, P15 )\n" +
				"L15: ( P16, P14 )\n" +
				"L16: ( P2, P3 )\n" +
				"L17: ( P4, P5 )\n" +
				"L18: ( P8, P7 )\n" +
				"L19: ( P9, P10 )\n" +
				"Intersections:\n" +
				"I0 ( L4, L6 )\n" +
				"I2 ( L4, L7 )\n" +
				"I1 ( L5, L6 )\n" +
				"I3 ( L5, L7 )\n" +
				"I4 ( L8, L9 )\n" +
				"I5 ( L10, L11 )\n", knotDiagram.toString());
		
		knotDiagram.setIntersectionType(knotDiagram.getLines().get(4), knotDiagram.getLines().get(6));
		knotDiagram.setIntersectionType(knotDiagram.getLines().get(4), knotDiagram.getLines().get(7));
		knotDiagram.setIntersectionType(knotDiagram.getLines().get(6), knotDiagram.getLines().get(5));
		
		TestCase.assertEquals("test13_3: ", "Points:\n" +
				"P0: ( 2, 2 )\n" +
				"P1: ( 3, 2 )\n" +
				"P2: ( 3, 3 )\n" +
				"P3: ( 3, 4 )\n" +
				"P4: ( 4, 4 )\n" +
				"P5: ( 4, 3 )\n" +
				"P6: ( 4, 2 )\n" +
				"P7: ( 5, 2 )\n" +
				"P8: ( 5, 4 )\n" +
				"P9: ( 5, 7 )\n" +
				"P10: ( 5, 9 )\n" +
				"P11: ( 3, 9 )\n" +
				"P12: ( 2, 9 )\n" +
				"P13: ( 3, 8 )\n" +
				"P14: ( 4, 8 )\n" +
				"P15: ( 3, 7 )\n" +
				"P16: ( 4, 7 )\n" +
				"P17: ( 2, 7 )\n" +
				"P18: ( 2, 4 )\n" +
				"P19: ( 4, 9 )\n" +
				"Lines:\n" +
				"L0: ( P1, P0 )\n" +
				"L1: ( P7, P6 )\n" +
				"L2: ( P11, P12 )\n" +
				"L3: ( P10, P19 )\n" +
				"L4: ( P17, P4 )\n" +
				"    I0 I2 \n" +
				"L5: ( P15, P8 )\n" +
				"    I1 I3 \n" +
				"L6: ( P18, P16 )\n" +
				"    I0 I1 \n" +
				"L7: ( P3, P9 )\n" +
				"    I2 I3 \n" +
				"L8: ( P14, P11 )\n" +
				"    I4 \n" +
				"L9: ( P19, P13 )\n" +
				"    I4 \n" +
				"L10: ( P6, P2 )\n" +
				"    I5 \n" +
				"L11: ( P5, P1 )\n" +
				"    I5 \n" +
				"L12: ( P0, P18 )\n" +
				"L13: ( P12, P17 )\n" +
				"L14: ( P13, P15 )\n" +
				"L15: ( P16, P14 )\n" +
				"L16: ( P2, P3 )\n" +
				"L17: ( P4, P5 )\n" +
				"L18: ( P8, P7 )\n" +
				"L19: ( P9, P10 )\n" +
				"Intersections:\n" +
				"I0 ( L4, L6 )\n" +
				"I2 ( L4, L7 )\n" +
				"I3 ( L5, L7 )\n" +
				"I1 ( L6, L5 )\n" +
				"I4 ( L8, L9 )\n" +
				"I5 ( L10, L11 )\n", knotDiagram.toString());
		
		knotDiagram.setIntersectionType(knotDiagram.getLines().get(5), knotDiagram.getLines().get(7));
		knotDiagram.setIntersectionType(knotDiagram.getLines().get(10), knotDiagram.getLines().get(11));
		knotDiagram.setIntersectionType(knotDiagram.getLines().get(9), knotDiagram.getLines().get(8));
		
		TestCase.assertEquals("test13_4: ", "Points:\n" +
				"P0: ( 2, 2 )\n" +
				"P1: ( 3, 2 )\n" +
				"P2: ( 3, 3 )\n" +
				"P3: ( 3, 4 )\n" +
				"P4: ( 4, 4 )\n" +
				"P5: ( 4, 3 )\n" +
				"P6: ( 4, 2 )\n" +
				"P7: ( 5, 2 )\n" +
				"P8: ( 5, 4 )\n" +
				"P9: ( 5, 7 )\n" +
				"P10: ( 5, 9 )\n" +
				"P11: ( 3, 9 )\n" +
				"P12: ( 2, 9 )\n" +
				"P13: ( 3, 8 )\n" +
				"P14: ( 4, 8 )\n" +
				"P15: ( 3, 7 )\n" +
				"P16: ( 4, 7 )\n" +
				"P17: ( 2, 7 )\n" +
				"P18: ( 2, 4 )\n" +
				"P19: ( 4, 9 )\n" +
				"Lines:\n" +
				"L0: ( P1, P0 )\n" +
				"L1: ( P7, P6 )\n" +
				"L2: ( P11, P12 )\n" +
				"L3: ( P10, P19 )\n" +
				"L4: ( P17, P4 )\n" +
				"    I0 I2 \n" +
				"L5: ( P15, P8 )\n" +
				"    I1 I3 \n" +
				"L6: ( P18, P16 )\n" +
				"    I0 I1 \n" +
				"L7: ( P3, P9 )\n" +
				"    I2 I3 \n" +
				"L8: ( P14, P11 )\n" +
				"    I4 \n" +
				"L9: ( P19, P13 )\n" +
				"    I4 \n" +
				"L10: ( P6, P2 )\n" +
				"    I5 \n" +
				"L11: ( P5, P1 )\n" +
				"    I5 \n" +
				"L12: ( P0, P18 )\n" +
				"L13: ( P12, P17 )\n" +
				"L14: ( P13, P15 )\n" +
				"L15: ( P16, P14 )\n" +
				"L16: ( P2, P3 )\n" +
				"L17: ( P4, P5 )\n" +
				"L18: ( P8, P7 )\n" +
				"L19: ( P9, P10 )\n" +
				"Intersections:\n" +
				"I0 ( L4, L6 )\n" +
				"I2 ( L4, L7 )\n" +
				"I3 ( L5, L7 )\n" +
				"I1 ( L6, L5 )\n" +
				"I4 ( L9, L8 )\n" +
				"I5 ( L10, L11 )\n", knotDiagram.toString());
	}
	
	public void test14(){
		IKnotDiagram knotDiagram = new KnotDiagram();
		
		List<IPoint> list = knotDiagram.getPoints();
		
		knotDiagram.addPoint(2, 2);
		knotDiagram.addPoint(3, 2);
		knotDiagram.addPoint(3, 3);
		knotDiagram.addPoint(3, 4);
		knotDiagram.addPoint(4, 4);
		knotDiagram.addPoint(4, 3);
		knotDiagram.addPoint(4, 2);
		knotDiagram.addPoint(5, 2);
		knotDiagram.addPoint(5, 4);
		knotDiagram.addPoint(5, 7);
		knotDiagram.addPoint(5, 9);
		knotDiagram.addPoint(3, 9);
		knotDiagram.addPoint(2, 9);
		knotDiagram.addPoint(3, 8);
		knotDiagram.addPoint(4, 8);
		knotDiagram.addPoint(3, 7);
		knotDiagram.addPoint(4, 7);
		knotDiagram.addPoint(2, 7);
		knotDiagram.addPoint(2, 4);
		knotDiagram.addPoint(4, 9);
		
		TestCase.assertEquals("test14_1: ", "Points:\n" +
				"P0: ( 2, 2 )\n" +
				"P1: ( 3, 2 )\n" +
				"P2: ( 3, 3 )\n" +
				"P3: ( 3, 4 )\n" +
				"P4: ( 4, 4 )\n" +
				"P5: ( 4, 3 )\n" +
				"P6: ( 4, 2 )\n" +
				"P7: ( 5, 2 )\n" +
				"P8: ( 5, 4 )\n" +
				"P9: ( 5, 7 )\n" +
				"P10: ( 5, 9 )\n" +
				"P11: ( 3, 9 )\n" +
				"P12: ( 2, 9 )\n" +
				"P13: ( 3, 8 )\n" +
				"P14: ( 4, 8 )\n" +
				"P15: ( 3, 7 )\n" +
				"P16: ( 4, 7 )\n" +
				"P17: ( 2, 7 )\n" +
				"P18: ( 2, 4 )\n" +
				"P19: ( 4, 9 )\n" +
				"Lines:\n" +
				"Intersections:\n", knotDiagram.toString());
		
		knotDiagram.connectPoints(list.get(0), list.get(1));
		knotDiagram.connectPoints(list.get(6), list.get(7));
		knotDiagram.connectPoints(list.get(12), list.get(11));
		knotDiagram.connectPoints(list.get(19), list.get(10));
		knotDiagram.connectPoints(list.get(17), list.get(4));
		knotDiagram.connectPoints(list.get(15), list.get(8));
		knotDiagram.connectPoints(list.get(16), list.get(18));
		knotDiagram.connectPoints(list.get(9), list.get(3));
		knotDiagram.connectPoints(list.get(11), list.get(14));
		knotDiagram.connectPoints(list.get(19), list.get(13));
		knotDiagram.connectPoints(list.get(2), list.get(6));
		knotDiagram.connectPoints(list.get(5), list.get(1));
		knotDiagram.connectPoints(list.get(0), list.get(18));
		knotDiagram.connectPoints(list.get(12), list.get(17));
		knotDiagram.connectPoints(list.get(13), list.get(15));
		knotDiagram.connectPoints(list.get(16), list.get(14));
		knotDiagram.connectPoints(list.get(3), list.get(2));
		knotDiagram.connectPoints(list.get(5), list.get(4));
		knotDiagram.connectPoints(list.get(8), list.get(7));
		knotDiagram.connectPoints(list.get(9), list.get(10));
		
		//System.out.println(knotDiagram);
		TestCase.assertEquals("test14_2: ", "Points:\n" +
				"P0: ( 2, 2 )\n" +
				"P1: ( 3, 2 )\n" +
				"P2: ( 3, 3 )\n" +
				"P3: ( 3, 4 )\n" +
				"P4: ( 4, 4 )\n" +
				"P5: ( 4, 3 )\n" +
				"P6: ( 4, 2 )\n" +
				"P7: ( 5, 2 )\n" +
				"P8: ( 5, 4 )\n" +
				"P9: ( 5, 7 )\n" +
				"P10: ( 5, 9 )\n" +
				"P11: ( 3, 9 )\n" +
				"P12: ( 2, 9 )\n" +
				"P13: ( 3, 8 )\n" +
				"P14: ( 4, 8 )\n" +
				"P15: ( 3, 7 )\n" +
				"P16: ( 4, 7 )\n" +
				"P17: ( 2, 7 )\n" +
				"P18: ( 2, 4 )\n" +
				"P19: ( 4, 9 )\n" +
				"Lines:\n" +
				"L0: ( P1, P0 )\n" +
				"L1: ( P7, P6 )\n" +
				"L2: ( P11, P12 )\n" +
				"L3: ( P10, P19 )\n" +
				"L4: ( P17, P4 )\n" +
				"    I0 I2 \n" +
				"L5: ( P15, P8 )\n" +
				"    I1 I3 \n" +
				"L6: ( P18, P16 )\n" +
				"    I0 I1 \n" +
				"L7: ( P3, P9 )\n" +
				"    I2 I3 \n" +
				"L8: ( P14, P11 )\n" +
				"    I4 \n" +
				"L9: ( P19, P13 )\n" +
				"    I4 \n" +
				"L10: ( P6, P2 )\n" +
				"    I5 \n" +
				"L11: ( P5, P1 )\n" +
				"    I5 \n" +
				"L12: ( P0, P18 )\n" +
				"L13: ( P12, P17 )\n" +
				"L14: ( P13, P15 )\n" +
				"L15: ( P16, P14 )\n" +
				"L16: ( P2, P3 )\n" +
				"L17: ( P4, P5 )\n" +
				"L18: ( P8, P7 )\n" +
				"L19: ( P9, P10 )\n" +
				"Intersections:\n" +
				"I0 ( L4, L6 )\n" +
				"I2 ( L4, L7 )\n" +
				"I1 ( L5, L6 )\n" +
				"I3 ( L5, L7 )\n" +
				"I4 ( L8, L9 )\n" +
				"I5 ( L10, L11 )\n", knotDiagram.toString());
		
		knotDiagram.changeLineDirection(knotDiagram.getLines().get(17));
		
		TestCase.assertEquals("test14_3: ", "Points:\n" +
				"P0: ( 2, 2 )\n" +
				"P1: ( 3, 2 )\n" +
				"P2: ( 3, 3 )\n" +
				"P3: ( 3, 4 )\n" +
				"P4: ( 4, 4 )\n" +
				"P5: ( 4, 3 )\n" +
				"P6: ( 4, 2 )\n" +
				"P7: ( 5, 2 )\n" +
				"P8: ( 5, 4 )\n" +
				"P9: ( 5, 7 )\n" +
				"P10: ( 5, 9 )\n" +
				"P11: ( 3, 9 )\n" +
				"P12: ( 2, 9 )\n" +
				"P13: ( 3, 8 )\n" +
				"P14: ( 4, 8 )\n" +
				"P15: ( 3, 7 )\n" +
				"P16: ( 4, 7 )\n" +
				"P17: ( 2, 7 )\n" +
				"P18: ( 2, 4 )\n" +
				"P19: ( 4, 9 )\n" +
				"Lines:\n" +
				"L0: ( P0, P1 )\n" +
				"L1: ( P7, P6 )\n" +
				"L2: ( P12, P11 )\n" +
				"L3: ( P10, P19 )\n" +
				"L4: ( P4, P17 )\n" +
				"    I0 I2 \n" +
				"L5: ( P15, P8 )\n" +
				"    I1 I3 \n" +
				"L6: ( P16, P18 )\n" +
				"    I0 I1 \n" +
				"L7: ( P3, P9 )\n" +
				"    I2 I3 \n" +
				"L8: ( P11, P14 )\n" +
				"    I4 \n" +
				"L9: ( P19, P13 )\n" +
				"    I4 \n" +
				"L10: ( P6, P2 )\n" +
				"    I5 \n" +
				"L11: ( P1, P5 )\n" +
				"    I5 \n" +
				"L12: ( P18, P0 )\n" +
				"L13: ( P17, P12 )\n" +
				"L14: ( P13, P15 )\n" +
				"L15: ( P14, P16 )\n" +
				"L16: ( P2, P3 )\n" +
				"L17: ( P5, P4 )\n" +
				"L18: ( P8, P7 )\n" +
				"L19: ( P9, P10 )\n" +
				"Intersections:\n" +
				"I0 ( L4, L6 )\n" +
				"I2 ( L4, L7 )\n" +
				"I1 ( L5, L6 )\n" +
				"I3 ( L5, L7 )\n" +
				"I4 ( L8, L9 )\n" +
				"I5 ( L10, L11 )\n", knotDiagram.toString());
		
		knotDiagram.changeLineDirection(knotDiagram.getLines().get(1));
		
		TestCase.assertEquals("test14_3: ", "Points:\n" +
				"P0: ( 2, 2 )\n" +
				"P1: ( 3, 2 )\n" +
				"P2: ( 3, 3 )\n" +
				"P3: ( 3, 4 )\n" +
				"P4: ( 4, 4 )\n" +
				"P5: ( 4, 3 )\n" +
				"P6: ( 4, 2 )\n" +
				"P7: ( 5, 2 )\n" +
				"P8: ( 5, 4 )\n" +
				"P9: ( 5, 7 )\n" +
				"P10: ( 5, 9 )\n" +
				"P11: ( 3, 9 )\n" +
				"P12: ( 2, 9 )\n" +
				"P13: ( 3, 8 )\n" +
				"P14: ( 4, 8 )\n" +
				"P15: ( 3, 7 )\n" +
				"P16: ( 4, 7 )\n" +
				"P17: ( 2, 7 )\n" +
				"P18: ( 2, 4 )\n" +
				"P19: ( 4, 9 )\n" +
				"Lines:\n" +
				"L0: ( P0, P1 )\n" +
				"L1: ( P6, P7 )\n" +
				"L2: ( P12, P11 )\n" +
				"L3: ( P19, P10 )\n" +
				"L4: ( P4, P17 )\n" +
				"    I0 I2 \n" +
				"L5: ( P8, P15 )\n" +
				"    I1 I3 \n" +
				"L6: ( P16, P18 )\n" +
				"    I0 I1 \n" +
				"L7: ( P9, P3 )\n" +
				"    I2 I3 \n" +
				"L8: ( P11, P14 )\n" +
				"    I4 \n" +
				"L9: ( P13, P19 )\n" +
				"    I4 \n" +
				"L10: ( P2, P6 )\n" +
				"    I5 \n" +
				"L11: ( P1, P5 )\n" +
				"    I5 \n" +
				"L12: ( P18, P0 )\n" +
				"L13: ( P17, P12 )\n" +
				"L14: ( P15, P13 )\n" +
				"L15: ( P14, P16 )\n" +
				"L16: ( P3, P2 )\n" +
				"L17: ( P5, P4 )\n" +
				"L18: ( P7, P8 )\n" +
				"L19: ( P10, P9 )\n" +
				"Intersections:\n" +
				"I0 ( L4, L6 )\n" +
				"I2 ( L4, L7 )\n" +
				"I1 ( L5, L6 )\n" +
				"I3 ( L5, L7 )\n" +
				"I4 ( L8, L9 )\n" +
				"I5 ( L10, L11 )\n", knotDiagram.toString());
		
		knotDiagram.changeLineDirection(knotDiagram.getLines().get(13));
		
		TestCase.assertEquals("test14_5: ", "Points:\n" +
				"P0: ( 2, 2 )\n" +
				"P1: ( 3, 2 )\n" +
				"P2: ( 3, 3 )\n" +
				"P3: ( 3, 4 )\n" +
				"P4: ( 4, 4 )\n" +
				"P5: ( 4, 3 )\n" +
				"P6: ( 4, 2 )\n" +
				"P7: ( 5, 2 )\n" +
				"P8: ( 5, 4 )\n" +
				"P9: ( 5, 7 )\n" +
				"P10: ( 5, 9 )\n" +
				"P11: ( 3, 9 )\n" +
				"P12: ( 2, 9 )\n" +
				"P13: ( 3, 8 )\n" +
				"P14: ( 4, 8 )\n" +
				"P15: ( 3, 7 )\n" +
				"P16: ( 4, 7 )\n" +
				"P17: ( 2, 7 )\n" +
				"P18: ( 2, 4 )\n" +
				"P19: ( 4, 9 )\n" +
				"Lines:\n" +
				"L0: ( P1, P0 )\n" +
				"L1: ( P6, P7 )\n" +
				"L2: ( P11, P12 )\n" +
				"L3: ( P19, P10 )\n" +
				"L4: ( P17, P4 )\n" +
				"    I0 I2 \n" +
				"L5: ( P8, P15 )\n" +
				"    I1 I3 \n" +
				"L6: ( P18, P16 )\n" +
				"    I0 I1 \n" +
				"L7: ( P9, P3 )\n" +
				"    I2 I3 \n" +
				"L8: ( P14, P11 )\n" +
				"    I4 \n" +
				"L9: ( P13, P19 )\n" +
				"    I4 \n" +
				"L10: ( P2, P6 )\n" +
				"    I5 \n" +
				"L11: ( P5, P1 )\n" +
				"    I5 \n" +
				"L12: ( P0, P18 )\n" +
				"L13: ( P12, P17 )\n" +
				"L14: ( P15, P13 )\n" +
				"L15: ( P16, P14 )\n" +
				"L16: ( P3, P2 )\n" +
				"L17: ( P4, P5 )\n" +
				"L18: ( P7, P8 )\n" +
				"L19: ( P10, P9 )\n" +
				"Intersections:\n" +
				"I0 ( L4, L6 )\n" +
				"I2 ( L4, L7 )\n" +
				"I1 ( L5, L6 )\n" +
				"I3 ( L5, L7 )\n" +
				"I4 ( L8, L9 )\n" +
				"I5 ( L10, L11 )\n", knotDiagram.toString());
		
	}
	
}
